(function() {
    var root = this;

    var SM = root.ScheduleMaker = {};

    SM.minSunAmTeachers = 3;
    SM.minSunPmTeachers = 2;
    SM.minWedTeachers = 2;

    SM.views = [];

    SM.ServiceView = function(element) {

        // #### Set properties
        this.el = element;
        this.$el = $(element);
        var thisView = this;

        this.$el.attr('viewId', SM.views.length)
        SM.views.push(thisView);

        // #### Set methods
        this.findTeacherMinimum = function($ulEl) {
            if ($ulEl.parents('.am').length > 0) {
                return SM.minSunAmTeachers; }
            else if ($ulEl.parents('.pm').length > 0) {
                return SM.minSunPmTeachers; }
            else return SM.minWedTeachers; };

        this.volunteersAdded = function(event) {
            // event.target should refer to the ul receiving the item.
            var $ulEl = $(event.target);

            if ($ulEl.find('li.teacher-needed').length > 0) {
                $ulEl.find('li.teacher-needed').first().remove();
                $ulEl.sortable("refresh"); } };

        this.updatePlaceholders = function(event) {
            // event.target should refer to the ul receiving the item.
            var $ulEl = $(event.target);

            while ($ulEl.find('li').length <
                this.findTeacherMinimum($ulEl)) {
                var $li = $(document.createElement('li'));
                $li.addClass('teacher-needed');
                $li.text('teacher needed');
                $li.on('dblclick', this.editPlaceholder);
                $ulEl.append($li[0]);
                $ulEl.sortable("refresh"); } };

        this.deleteName = function (event) { 
            // event.target is the ul > li > span element containing the 'X'
            var ulEl = $(event.target).closest('ul')[0];
            $(event.target).closest('li').remove();
            this.updatePlaceholders({target: ulEl})};

        this.copyName = function (event) {
            // event.target is the ul > li > span element containing the 'C'
            var $liEl = $(event.target).closest('li');
            var $ulEl = $liEl.closest('ul');
            $ulEl.append($liEl.clone(true));
            this.volunteersAdded({target: $ulEl[0]}); };

        this.editName = function(event) {
            // event.target is the li > span.name element that was clicked.
            var $spanEl = $(event.target);
            var $liEl = $spanEl.parent();
            $liEl.find('input.name').val($spanEl.text());
            $liEl.addClass('editing');
            $liEl.find('input.name').focus(); };

        this.saveName = function(event) {
            // event.target is the li > input.name element that was blurred.
            var $inputEl = $(event.target);
            var $liEl = $inputEl.parent();
            $liEl.find('span.name').text($inputEl.val());
            $liEl.removeClass('editing'); };

        this.editPlaceholder = function(event) {
            var liEl = this.newName(event);
            this.editName({target: $(liEl).find('span.name')[0]}); };

        this.newName = function(event) {
            var $ulEl = $(event.target).closest('ul');
            var $liEl = $(document.createElement('li'));
            $liEl.append('<span class=name></span><input class=name>' +
                         '<span class=clone>C</span><span class=remove>X</span>');
            $liEl.find('span.name').on('dblclick', this.editName);
            $liEl.find('input.name').on('blur', this.saveName);
            $liEl.find('.remove').on('click', this.deleteName);
            $liEl.find('.clone').on('click', this.copyName);

            $ulEl.append($liEl[0]);

            this.volunteersAdded({target: $ulEl[0]});
            $ulEl.sortable("refresh");
            return $liEl[0]; };

        var funs = ['newName', 'saveName', 'editName', 'deleteName',
            'copyName', 'editPlaceholder', 'volunteersAdded',
            'updatePlaceholders'];
        funs.forEach(function(funName) {
            thisView[funName] = thisView[funName].bind(thisView); });

        // #### Initialize the existing view objects.
        var $volunteerLists = this.$el.find('ul');
        $volunteerLists.sortable({
            connectWith: 'ul',
            cancel: 'li.teacher-needed' });

        // updatePlaceholders adds place-holders if there are not enough
        // teachers. We call this once during initialization to pad out the
        // lists to the minimums
        $volunteerLists.each(function(idx) {
            thisView.updatePlaceholders({target: this}); });

        // #### Event handlers 

        // Handlers to refresh the missing teacher spots when someone is
        // drag-and-dropped.
        $volunteerLists.on("sortreceive", this.volunteersAdded);
        $volunteerLists.on("sortremove", this.updatePlaceholders);

        // Handler to remove names when the 'X' is clicked.
        this.$el.find('.remove').on('click', this.deleteName);

        // Handler to edit names when they are double-clicked.
        this.$el.find('span.name').on('dblclick', this.editName);

        // Handler to save the editing changes back to the li
        this.$el.find('input.name').on('blur', this.saveName);
    };

    SM.saveSchedule = function() {

        // Create the JSON object to serialize to the file.
        var data = {};

        // Save the date.
        data.date = $('input[type=date]').val();

        // Gather the sunday data
        data.sundays = {};
        $('.sundays > div').each(function() {
            var divEl = this;
            var $divEl = $(this);

            data.sundays[divEl.className] = {};

            // Save each of the service data.
            $divEl.find('ul').each(function() {
                var ulEl = this;
                var $ulEl = $(this);

                var roomName = 'pm';
                if ($ulEl.parents('.toddlers').length > 0) roomName = 'toddlers';
                else if ($ulEl.parents('.infants').length > 0) roomName = 'infants';

                data.sundays[divEl.className][roomName] = 
                    $ulEl.find('span.name').map(function() {
                        return $(this).text(); }).get();
            });
        });

        // Gather Wednesday data
        data.wednesdays = {};
        $('.wednesdays > div').each(function() {
            var divEl = this;
            var $divEl = $(this);

            // Save the service data.
            data.wednesdays[divEl.className] = $divEl.find('span.name').map(
                function() { return $(this).text(); }).get(); });
                
        var jsonData = JSON.stringify(data);
        var jsonBlob = new Blob([jsonData], {type: 'text/plain'});

        // Save the file to disk by presenting it as a download to the user.
        var linkEl = document.createElement('a');
        var $linkEl = $(linkEl);
        linkEl.download = "nursery-schedule-" + $('input[type=date]').val();
        linkEl.innerHTML = "Download Schedule Data"

        if (window.webkitURL != null) {
            linkEl.href = window.webkitURL.createObjectURL(jsonBlob); }
        else {
            linkEl.href = window.URL.createObjectURL(jsonBlob);
            $(linkEl).on("click", function (e) { $(e.target).remove(); });
            $(linkEl).css('display', 'none');
            $('body').append(linkEl); }
        linkEl.click(); };

    SM.hideLoadDialog = function(e) {
        if (e.target == $('#fileLoad')[0]) return;
        $('.load-dialog').fadeOut(); };

    SM.showLoadDialog = function() { $('.load-dialog').fadeIn(); };

    SM.loadSchedule = function() {
        // Remove exsting data.
        $('.sundays ul li, .wednesdays ul li').remove()

        var fileName = $('#fileLoad')[0].files[0];
        var fileReader = new FileReader();

        fileReader.onload = function(e) {
            var fileText = e.target.result;
            var data = JSON.parse(fileText);

            // Restore the date.
            $('input[type=date]').val(data.date);

            // Restore Sunday services
            for(var day in data.sundays) {
                for(var roomName in data.sundays[day]) {
                    var $divEl = $('.sundays .' + day);
                    var $ulEl = $divEl.find('.' + roomName + ' ul');
                    var view = SM.views[parseInt($divEl.attr('viewId'))];
                    data.sundays[day][roomName].forEach(function(personName) {
                        var liEl = view.newName({target: $ulEl[0]});
                        $(liEl).find('span.name').text(personName); });
                    view.updatePlaceholders({target: $ulEl[0]}); } }

            // Restore Wednesday services
            for(var day in data.wednesdays) {
                var $divEl = $('.wednesdays .' + day);
                var $ulEl = $divEl.find('ul');
                var view = SM.views[parseInt($divEl.attr('viewId'))];
                data.wednesdays[day].forEach(function(personName) {
                    var liEl = view.newName({target: $ulEl[0]});
                    $(liEl).find('span.name').text(personName); });
                view.updatePlaceholders({target: $ulEl[0]}); } }
                
        fileReader.readAsText(fileName, "UTF-8");
    };

    SM.publicShedule = function() {
        // TODO
    };

    SM.init = function() {
        // Create our ServiceViews
        $('.sundays > div, .wednesdays > div').each(function(idx, elem) {
            var view = new SM.ServiceView(elem); });
        
        // Wire up our toolbar
        $('.save').on('click', SM.saveSchedule);
        $('.load').on('click', SM.showLoadDialog);
        $('.publish').on('click', SM.publishSchedule);
        $('.load-dialog').on('click', SM.hideLoadDialog);
        $('.load-dialog a').on('click', SM.loadSchedule);
    };

}).call(this);
